/* fileindex.c -- Copyright 1989, 1990, 1993 Liam R. Quin.  All Rights Reserved.
 * This code is NOT in the public domain.
 * See the file COPYRIGHT for full details.
 */

/* A simple program to give information about one or more files about
 * which information is stored in the lq-text database.
 */

static char *Revision = "@(#) $Id: lqfile.c,v 1.6 93/11/14 15:27:15 lee Exp $";

#include "globals.h" /* defines and declarations for database filenames */

#include "error.h"

#include <stdio.h>
#include <sys/types.h>
#include <malloc.h>
#include "emalloc.h"
#include "fileinfo.h"

/* The position of the \n in the 26-char string returned by ctime(3): */
#define DATENEWLINE 24

char *progname;
int AsciiTrace = 0;

/** System calls and library functions used in this file: **/

/** Unix System calls: **/
extern void exit();
/** System Library Functions: **/

/** external lqtext functions: **/
extern void cleanupdb(), SetDefaults();
int SaveFileInfo(), GetFilterType();
#ifndef efree
 extern void efree();
#endif
/** Functions defined within this file: **/
static void AddInfo(), AllInfo();
static void DisplayFileInfo(), PrintFIDInfo(), PrintInfo();

int AllFiles = 0;
int ListMode = 0;
int AddFiles = 0;

int
main(argc, argv)
    int argc;
    char *argv[];
{
    extern int optind, getopt();
    /** extern char *optarg; (unused at the moment) **/
    int ch;
    int ErrorFlag = 0;
    int FIDMode = 0;

    progname = argv[0];

    SetDefaults(argc, argv);

    /* All programs take Zz:Vv */
    while ((ch = getopt(argc, argv, "Zz:VvAaFlx")) != EOF) {
	switch (ch) {
	case 'z':
	case 'Z':
	    break; /* done by SetDefaults(); */
	case 'V':
	    fprintf(stderr, "%s version %s\n", progname, Revision);
	    break;
	case 'v':
	    AsciiTrace = 1;
	    break;
	case 'F':
	    FIDMode = 1;
	    break; case 'A':
	    AddFiles = 1;
	    break;
	case 'a':
	    AllFiles = 1;
	    break;
	case 'l':
	    ListMode = 1;
	    break;
	case 'x':
	    ErrorFlag = (-1);
	    break;
	case '?':
	    ErrorFlag = 1;
	    break;
	}
    }

    /* Normally put call to lrqError here to give a helpful message,
     * but not yet ready to ship the error handling package, sorry
     */
    if (ErrorFlag) {
	fprintf(stderr, "%s: usage: %s [options] [files]\n",progname,progname);
	fprintf(stderr, "%s: options are:\n", progname);
	fputs("\
	-c file -- treat the named file as a list of common words\n\
	-d dir	-- use the lq-text database in the directory \"dir\"\n\
	-F      -- treat file names as numeric FIDs instead\n\
	-l	-- list mode: no header output or lines drawn\n\
	-s	-- show the list of saved files\n\
	-t N	-- set trace level to N [default: 0]\n\
	-V	-- print version information\n\
	-v	-- be verbose (same as -t 1)\n\
	-x	-- print this explanation\n\
\n\
In addition, if no files are given, the following are understood:\n\
	-A	-- add the named files to the list of known files\n\
	-a	-- list information about all files\n", stderr);
	exit((ErrorFlag > 0) ? 1 : 0);
    }

    if (AllFiles && AddFiles) {
	fprintf(stderr, "%s: do not use both -a and -A options\n", progname);
	fprintf(stderr, "\tuse %s -x for further explanation.\n", progname);
	exit(1);
    }

    if (optind >= argc && !AllFiles && !AddFiles) {
	Error(E_USAGE|E_FATAL|E_XHINT,
	    "You must either give the -a option or specify files to list"
	);
    }

    if (!AddFiles && !ListMode) {
	printf("%-7.7s | T | %-20.20s | %s\n",
		"FID", "Date Last indexed", "Current Location");
	puts(
"========|===|======================|=========================================="
	);
    }
    if (AllFiles) {
	AllInfo();
    } else {
	if (AddFiles) {
	    extern lqWriteAccess();

	    lqWriteAccess();
	}

	while (optind < argc) {
	    if (AddFiles) {
		AddInfo(argv[optind++]);
	    } else {
		if (FIDMode) {
		    extern long atol();
		    t_FID FID;

		    if ((FID = atol(argv[optind])) == (t_FID) 0) {
			Error(E_WARN,
			    "Invalid FID %s; FIDs must be numeric, > 0L",
			    argv[optind]
			);
		    } else {
			PrintFIDInfo(FID);
		    }
		} else {
		    PrintInfo(argv[optind]);
		}
		++optind;
	    }
	}
    }
    cleanupdb(); /* close dbm files */
    exit(0);
    /*NOTREACHED*/
    return 1; /* for lint and gcc... */
}

static void
PrintInfo(Name)
    char *Name;
{
    extern t_FID Name2FID();
    t_FID FID;

    if ((FID = Name2FID(Name)) == (t_FID) 0) {
	Error(E_WARN,
	    "No FID available for filename: %s",
	    Name
	);
    } else {
	PrintInfo(FID);
    }
}

static void
PrintFIDInfo(FID)
    t_FID FID;
{
    extern t_FileInfo *GetFileInfo();
    t_FileInfo *FileInfo;

    /* get info from the list */
    if ((FileInfo = GetFileInfo(FID)) == (t_FileInfo *) 0) {
	Error(E_WARN,
	    "No Index information for FID: %ld",
	    FID
	);
    } else {
	DisplayFileInfo(FileInfo);
    }
}

static void
DisplayFileInfo(FileInfo)
    t_FileInfo *FileInfo;
{
    extern char *ctime();
    char *DateString;

    DateString = ctime(&(FileInfo->Date));
    DateString[DATENEWLINE] = '\0'; /* delete the trailing newline */

    if (ListMode) {
	printf("%lu\t%d\t%s\t%s\n",
	FileInfo->FID, FileInfo->FilterType, &DateString[4], FileInfo->Name);
    } else {
	printf("%7lu | %d | %-20.20s | %s\n",
	    FileInfo->FID, FileInfo->FilterType, &DateString[4], FileInfo->Name);
    }
}

/**
Mon Sep 25 23:58:53 BST 1989
FID     | T | Date Last indexed    | Current Location
========|===|======================|===========================================
      1 | 0 | Sep 25 20:31:26 1989 | /usr2/liam/Bible/NT/John/john01.kjv
      2 | 0 | Sep 25 20:31:28 1989 | /usr2/liam/Bible/NT/John/john02.kjv
      3 | 0 | Sep 25 20:31:30 1989 | /usr2/liam/Bible/NT/John/john03.kjv
**/

static void
AllInfo()
{
    extern long GetMaxFID();
    extern t_FileInfo *GetFileInfo();

    t_FileInfo *FileInfo;
    long FID;
    long MaxFid = GetMaxFID();

    for (FID = 0L; FID <= MaxFid; FID++) {
	if ((FileInfo = GetFileInfo(FID)) != (t_FileInfo *) 0) {
	    DisplayFileInfo(FileInfo);
	    efree(FileInfo); /* NOTDONE use destroyfileinfo() */
	}
    }
    if (!ListMode) {
	printf("Max File Identifier is %lu\n", MaxFid);
    }
}

static void
AddInfo(FileName)
    char *FileName;
{
    extern time_t time();
    extern unsigned long GetNextFID();
    t_FileInfo FileInfo;

    FileInfo.Name = FileName;
    (void) time(&(FileInfo.Date));
    FileInfo.FID = GetNextFID();
    FileInfo.Stream = 0; /* force GetFilterType to use open()? */

    /* determine filter type */
    FileInfo.FilterType = GetFilterType(&FileInfo);

    printf("%d %s (type %d) -- %s\n",
	    FileInfo.FID,
	    FileInfo.Name,
	    FileInfo.FilterType,
	    SaveFileInfo(&FileInfo) == 0 ?
			    "Marked as having been indexed." :
			    "Failed to mark file as having been indexed."
    );
}
